@@ -1,5 +1,5 @@
 --[[
-Copyright (c) 2016, Vsevolod Stakhov <vsevolod@highsecure.ru>
+Copyright (c) 2022, Vsevolod Stakhov <vsevolod@rspamd.com>
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -41,7 +41,7 @@ local settings = {
     max_memory = 50 * 1024 * 1024, -- How many memory should be occupied before sending collection
     max_interval = 60, -- Maximum collection interval
   },
-  collect_garbage = false, -- Peform GC collection after sending the data
+  collect_garbage = false, -- Perform GC collection after sending the data
   check_timeout = 10.0, -- Periodic timeout
   timeout = 5.0,
   bayes_spam_symbols = {'BAYES_SPAM'},
@@ -988,6 +988,10 @@ local function get_last_removal_ago()
     f:close()
   end
 
+  if last_ts == nil or (last_ts + settings.retention.period) <= current_ts then
+    return write_ts_to_file() and 0
+  end
+
   if last_ts > current_ts then
     -- Clock skew detected, overwrite last_ts with current_ts and wait for the next
     -- retention period
@@ -996,10 +1000,6 @@ local function get_last_removal_ago()
     return write_ts_to_file() and -1
   end
 
-  if last_ts == nil or (last_ts + settings.retention.period) <= current_ts then
-    return write_ts_to_file() and 0
-  end
-
   return (last_ts + settings.retention.period) - current_ts
 end
 
@@ -1057,7 +1057,7 @@ local function clickhouse_maybe_send_data_periodic(cfg, ev_base, now)
 
     clickhouse_send_data(nil, ev_base, reason, saved_rows, saved_custom)
 
-    if settings.collect_garbadge then
+    if settings.collect_garbage then
       collectgarbage()
     end
   end
@@ -1484,7 +1484,7 @@ if opts then
         clickhouse_send_data(task, nil, 'final collection',
             saved_rows, saved_custom)
 
-        if settings.collect_garbadge then
+        if settings.collect_garbage then
           collectgarbage()
         end
       end
